package org.nakedobjects.nos.client.web.html;

import java.io.PrintWriter;

import org.nakedobjects.nos.client.web.component.Component;
import org.nakedobjects.nos.client.web.component.ImageLookup;
import org.nakedobjects.nos.client.web.component.ViewPane;


public class ViewDiv extends CompositeComponent implements ViewPane {
    private String iconName;
    private String objectId;
    private Component[] menu = new Component[0];
    private String title;
    private String[] messages = new String[0];
    private String[] warnings = new String[0];
    private String description;
 
    public void setIconName(String iconName) {
        this.iconName = iconName;
    }

    public void setLink(String objectId) {
        this.objectId = objectId;
    }
    
    public void setMenu(Component[] menu) {
        this.menu = menu;
    }

    public void setTitle(final String title, final String description) {
        this.title = title;
        this.description = description;
    }
    
    public void setWarningsAndMessages(String[] messages, String[] warnings) {
        this.messages = messages;
        this.warnings = warnings;
    }

    protected void writeAfter(PrintWriter writer) {
        writer.println("</div>");
        writer.println("</div>");
    }

    protected void writeBefore(PrintWriter writer) {
        writer.println("<div id=\"view\">");
        writeMessages(writer);
        writeHeader(writer);
        writeMenu(writer);
        writer.println("<div id=\"content\">");
    }

    private void writeMessages(PrintWriter writer) {
        if (warnings.length > 0 || messages.length > 0) {
            writer.print("<div class=\"message-header\">");
            for (int i = 0; i < warnings.length; i++) {
                writer.print("<div class=\"warning\">");
                writer.print(warnings[i]);
                writer.println("</div>");
            }
            for (int i = 0; i < messages.length; i++) {
                writer.print("<div class=\"message\">");
                writer.print(messages[i]);
                writer.println("</div>");
            }
            writer.print("</div>");
        }
    }

    private void writeMenu(PrintWriter writer) {
        writer.println("<div id=\"menu\">");
        writer.println("<h3>Actions</h3>");
        for (int j = 0; j < menu.length; j++) {
            menu[j].write(writer);
        }
        writer.println("</div>");
    }

    private void writeHeader(PrintWriter writer) {
        writer.print("<div class=\"header\"");
        if (description != null) {
            writer.print(" title=\"");
            writer.print(description);
            writer.print("\"");
        }
        writer.print(">");
        if(objectId != null) {
            writer.print("<a href=\"object.app?id=");
            writer.print(objectId);
            writer.print("\">");
        }
        if (iconName != null) {
            writer.print("<span class=\"header-icon\"><img src=\"images/");
            writer.print(ImageLookup.image(iconName));
            writer.print("\" alt=\"icon\" /></span>");
        }
        writer.print("<span class=\"header-text\"");
        writer.print(">");
        writer.print(title);
        writer.println("</span>");
        if(objectId != null) {
            writer.print("</a>");
        }
        writer.println("</div>");
    }
}

// Copyright (c) Naked Objects Group Ltd.
