
package org.nakedobjects.nos.client.web.object;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.component.ViewPane;
import org.nakedobjects.nos.client.web.request.Action;
import org.nakedobjects.nos.client.web.request.ActionException;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Request;


public abstract class AbstractObjectView implements Action {

    public final void execute(Request request, Context context, Page page) {
        String idString = request.getObjectId();
        NakedObject object = context.getMappedObject(idString);
        if (object == null) {
            throw new ActionException("No such object: " + idString);
        }

         page.setTitle(object.titleString());

        ViewPane content = page.getViewPane();
        content.setWarningsAndMessages(context.getMessages(), context.getWarnings());
        content.setTitle(object.titleString(), object.getSpecification().getDescription());
        content.setIconName(object.getIconName());
        

        if (addObjectToHistory()) {
            context.addObjectToHistory(idString);
        }
        
        context.purge();

        content.setMenu(Menu.menu(object, idString, context));

        String iconName = object.getIconName();
        if (iconName == null) {
            iconName = object.getSpecification().getShortName();
        }

        content.setIconName(iconName);

        String field = request.getProperty();
        doExecute(context, content, object, field);


        context.clearMessagesAndWarnings();
    }

    protected void doExecute(Context context, ViewPane content, NakedObject object, String field) {}

    protected boolean addObjectToHistory() {
        return false;
    }
}

// Copyright (c) Naked Objects Group Ltd.
