package org.nakedobjects.nos.client.web.object;

import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.component.Table;
import org.nakedobjects.nos.client.web.component.ViewPane;
import org.nakedobjects.nos.client.web.request.Action;
import org.nakedobjects.nos.client.web.request.ActionException;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Request;
import org.nakedobjects.nos.client.web.request.TableBuilder;


public class CollectionView implements Action {

    public final void execute(Request request, Context context, Page page) {
        String idString = request.getObjectId();
        NakedCollection collection = context.getMappedCollection(idString);
        if (collection == null) {
            throw new ActionException("No such collection: " + idString);
        }
        String titleString = collection.titleString();

        page.setTitle(titleString);

        ViewPane content = page.getViewPane();
        content.setWarningsAndMessages(context.getMessages(), context.getWarnings());
        content.setTitle(titleString, null);
        String iconName = collection.getIconName();
        if (iconName == null) {
            iconName = collection.getElementSpecification().getShortName();
        }
        content.setIconName(iconName);

        NakedObjectSpecification type = collection.getElementSpecification();
        if (collection.getElementSpecification().getStaticallyVisibleFields().length != 0) {
            Table table = TableBuilder.createTable(context, false, collection, titleString, type);
            content.add(table);
        } else {
            // TODO this should create a list component instead of a table
            Table table = TableBuilder.createTable(context, false, collection, titleString, type);
            content.add(table);
        }
        
        context.addCollectionCrumb(idString);
        
        context.addCollectionToHistory(idString);
        context.clearMessagesAndWarnings();
    }
   
    public String name() {
        return Request.COLLECTION_COMMAND;
    }

}

// Copyright (c) Naked Objects Group Ltd.
