package org.nakedobjects.nos.client.web.object;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nos.client.web.component.Component;
import org.nakedobjects.nos.client.web.request.Context;

import java.util.Vector;


public class Menu {

    public static Component[] menu(final NakedObject target, final String targetObjectId, final Context context) {
        NakedObjectSpecification specification = target.getSpecification();
        NakedObjectAction[] actions1 = specification.getObjectActions(NakedObjectAction.USER);
        NakedObjectAction[] actions2 = specification.getObjectActions(NakedObjectAction.EXPLORATION);
        NakedObjectAction[] actions = new NakedObjectAction[actions1.length + actions2.length];
        System.arraycopy(actions1, 0, actions, 0, actions1.length);
        System.arraycopy(actions2, 0, actions, actions1.length, actions2.length);
        Component[] menuItems = createMenu("Actions", target, actions, context, targetObjectId);
        return menuItems;
    }

    private static Component[] createMenu(
            final String menuName,
            final NakedObject target,
            final NakedObjectAction[] actions,
            final Context context,
            final String targetObjectId) {
        Vector v = new Vector();
        for (int j = 0; j < actions.length; j++) {
            NakedObjectAction action = actions[j];
            String name = action.getName();
            if (action.getActions().length > 0) {
                Component[] components = createMenu(name, target, action.getActions(), context, targetObjectId);
                v.addElement(context.getComponentFactory().createSubmenu(name, components));
            } else {
                // TODO change logic to show exploration actions during exploration
                if (action.isVisible(target) && action.isVisibleForSession() && action.getType() == NakedObjectAction.USER) {
                    String actionId = context.mapAction(action);
                    boolean collectParameters;
                    if (action.getParameterCount() == 0) {
                        collectParameters = false;
                    } else if (action.getParameterCount() == 1 && action.isContributedMethodWithSuitableParameter()
                            && target.getSpecification().isOfType(action.getParameterTypes()[0])) {
                        collectParameters = false;
                    } else {
                        collectParameters = true;
                    }
                    Consent usableInState = action.isUsable(target);
                    String status = usableInState.isVetoed() ? usableInState.getReason() : null;
                    Consent usablByUser = action.isUsable();
                    status = usablByUser.isVetoed() ? usablByUser.getReason() : status;
                    String description = action.getDescription();
                    v.addElement(context.getComponentFactory().createMenuItem(actionId, name, description, status,
                            action.getType(), collectParameters, targetObjectId));
                }
            }
        }

        Component[] items = new Component[v.size()];
        v.copyInto(items);
        return items;
    }

}

// Copyright (c) Naked Objects Group Ltd.
