package org.nakedobjects.nos.client.web.object;

import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.reflect.ValueAssociation;
import org.nakedobjects.noa.spec.Features;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.UnknownTypeException;
import org.nakedobjects.nos.client.web.component.Block;
import org.nakedobjects.nos.client.web.component.Component;
import org.nakedobjects.nos.client.web.component.ComponentFactory;
import org.nakedobjects.nos.client.web.component.ViewPane;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Request;


public class ObjectView extends AbstractObjectView {
    protected boolean addObjectToHistory() {
        return true;
    }

    protected void doExecute(Context context, ViewPane content, NakedObject object, String field) {
        String id = context.mapObject(object);
        createObjectView(context, object, content, id);
        NakedObjectSpecification specification = object.getSpecification();
        boolean immutable = Features.isAlwaysImmutable(specification)
                || (object.getResolveState().isPersistent() && Features.isImmutableOncePersisted(specification));
        boolean allFieldUneditable = true;
        NakedObjectField[] flds = specification.getFields();
        for (int i = 0; i < flds.length; i++) {
            if (flds[i].isUsable().isAllowed() || flds[i].isUsable(object).isAllowed()) {
                allFieldUneditable = false;
                break;
            }
        }
        if (!immutable && !allFieldUneditable) {
            content.add(context.getComponentFactory().createEditOption(id));
        }
        context.setObjectCrumb(object);
    }

    private void createObjectView(Context context, NakedObject object, ViewPane pane, String id) {
        NakedObjectSpecification specification = object.getSpecification();
        NakedObjectField[] visibleFields = specification.getDynamicallyVisibleFields(object);
        for (int i = 0; i < visibleFields.length; i++) {
            NakedObjectField field = visibleFields[i];

            ComponentFactory factory = context.getComponentFactory();
            Block fieldBlock = factory.createBlock("field", field.getDescription());
            fieldBlock.add(factory.createInlineBlock("label", field.getName(), null));
            fieldBlock.add(factory.createInlineBlock("separator", ":  ", null));

            NakedObjectsContext.getObjectPersistor().resolveField(object, field);
            if (field.isObject()) {
                NakedObject element = (NakedObject) field.get(object);
                if (element == null) {
                    Component entry = factory.createInlineBlock("value", "", null);
                    fieldBlock.add(entry);
                } else {
//                    NakedObjectsContext.getObjectPersistor().resolveImmediately(element);
                    String elementId = context.mapObject(element);
                    fieldBlock.add(factory.createObjectIcon(field, element, elementId, "value"));
                }
            } else if (field.isValue()) {
                NakedValue value = (NakedValue) field.get(object);
                boolean multiLine = ((ValueAssociation) field).getNoLines() > 1;
                boolean wrap = ((ValueAssociation) field).canWrap();
                Component entry = factory.createValueField(field, value, multiLine, wrap, false);
                fieldBlock.add(entry);
            } else if (field.isCollection()) {
                NakedCollection collection = (NakedCollection) field.get(object);
                Component link = factory.createCollectionIcon(field, collection, id);
                fieldBlock.add(link);
            } else {
                throw new UnknownTypeException(field);
            }

            pane.add(fieldBlock);
        }
    }

    public String name() {
        return Request.OBJECT_COMMAND;
    }

}

// Copyright (c) Naked Objects Group Ltd.
