package org.nakedobjects.nos.client.web.request;

import java.util.Enumeration;
import java.util.Vector;

import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.DebugString;


class CollectionMapping {
    private final Vector list = new Vector();
    private final NakedObjectSpecification elementSpecification;

    public CollectionMapping(final Context context, final NakedCollection collection) {
        elementSpecification = collection.getElementSpecification();
        Enumeration enumeration = collection.elements();
        while (enumeration.hasMoreElements()) {
            NakedObject element = (NakedObject) enumeration.nextElement();
            list.add(context.mapObject(element));
        }
    }

    public NakedCollection getCollection(Context context) {
        Vector elements = new Vector();
        Enumeration e = list.elements();
        while (e.hasMoreElements()) {
            String elementId = (String) e.nextElement();
            NakedObject adapter = context.getMappedObject(elementId);
            elements.add(adapter.getObject());
        }
        return (NakedCollection) NakedObjectsContext.getObjectLoader().createAdapterForCollection(elements, elementSpecification);
    }

    public NakedObjectSpecification getElementSpecification() {
        return elementSpecification;
    }

    public void debug(DebugString debug) {
        debug.indent();
        Enumeration e = list.elements();
        while (e.hasMoreElements()) {
            String elementId = (String) e.nextElement();
            debug.appendln(elementId);
        }
        debug.unindent();
    }
    
    public int hashCode() {
        return 1;
    }
    
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        
        if (obj instanceof CollectionMapping) {
            CollectionMapping other = (CollectionMapping)obj;
            if (other.elementSpecification == this.elementSpecification) {
                if (other.list.size() == this.list.size()) {
                    return true;
                }
            }
        }
        
        return false;
    }

    public boolean contains(String id) {
        Enumeration e = list.elements();
        while (e.hasMoreElements()) {
            String elementId = (String) e.nextElement();
            if (elementId.equals(id)) {
                return true;
            }
        }
        return false;
    }

    public Enumeration elements() {
        return list.elements();
    }
}

// Copyright (c) Naked Objects Group Ltd.
