package org.nakedobjects.nos.client.web.request;

import java.util.Enumeration;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.core.util.UnknownTypeException;
import org.nakedobjects.nos.client.web.component.Block;
import org.nakedobjects.nos.client.web.component.Component;


class ObjectHistory {
    private static final Logger LOG = Logger.getLogger(ObjectHistory.class);
    private static final int MAX = 8;
    private final Vector history = new Vector();

    
    private void add(HistoryEntry entry) {
        history.removeElement(entry);
        history.addElement(entry);
        LOG.debug("added to history: " + entry);
        if (history.size() > MAX) {
            LOG.debug("purging from history: " + history.elementAt(0));
            history.removeElementAt(0);
        }
    }

    public void debug(DebugString debug) {
        for (int i = history.size() - 1; i >= 0; i--) {
            HistoryEntry object = ((HistoryEntry) history.elementAt(i));
            debug.appendln(object.toString());
        }
    }

    public void listObjects(Context context, Block navigation) {
        Block taskBar = context.getComponentFactory().createBlock("history", null);
        taskBar.add(context.getComponentFactory().createHeading("History"));
        for (int i = history.size() - 1; i >= 0; i--) {
            HistoryEntry item = (HistoryEntry) history.elementAt(i);
            Component icon;
            if (item.type == HistoryEntry.OBJECT) {
                NakedObject object = context.getMappedObject(item.id);
                NakedObjectsContext.getObjectPersistor().resolveImmediately(object);
                icon = context.getComponentFactory().createObjectIcon(object, item.id, "item");
            } else if (item.type == HistoryEntry.COLLECTION) {
                NakedCollection object = context.getMappedCollection(item.id);
                icon = context.getComponentFactory().createCollectionIcon(object, item.id);
            } else {
                throw new UnknownTypeException(item);
            }
            taskBar.add(icon);
        }
        navigation.add(taskBar);
    }

    public void addObject(String idString) {
        add(new HistoryEntry(idString, HistoryEntry.OBJECT));
    }

    public void addCollection(String idString) {
        add(new HistoryEntry(idString, HistoryEntry.COLLECTION));
    }

    Enumeration elements() {
        return history.elements();
    }
}

class HistoryEntry {
    public static final int OBJECT = 1;
    public static final int COLLECTION = 2;

    final int type;
    final String id;

    public HistoryEntry(String idString, int type) {
        this.id = idString;
        this.type = type;
    }
    
    public int hashCode() {
        return id.hashCode();
    }
    
    public boolean equals(Object obj) {
        return ((HistoryEntry) obj).id.equals(id);
    }
    
    public String toString() {
        return (type == OBJECT ? "object " : "collection ")  + id;
    }
    
}

// Copyright (c) Naked Objects Group Ltd.
