package org.nakedobjects.nos.client.web.request;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedObjectLoader;
import org.nakedobjects.noa.adapter.Oid;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.Assert;
import org.nakedobjects.nof.core.util.DebugString;


class PersistentObjectMapping implements ObjectMapping {
    private final Oid oid;
    private final NakedObjectSpecification specification;
    private Version version;

    public PersistentObjectMapping(final NakedObject object) {
        oid = object.getOid();
        Assert.assertFalse("OID is for transient", oid.isTransient());
        Assert.assertFalse("adapter is for transient", object.getResolveState().isTransient());
        specification = object.getSpecification();
        version = object.getVersion();
    }

    public void debug(DebugString debug) {
        debug.appendln(specification.getFullName());
        if (version != null) {
            debug.appendln(version.toString());
        }
    }
    
    public Oid getOid() {
        return oid;
    }
    
    public NakedObject getObject() {
        return NakedObjectsContext.getObjectPersistor().getObject(oid, specification);
    }

    public int hashCode() {
        return oid.hashCode();
    }
    
    public boolean equals(Object obj) {
        if (obj.getClass() == PersistentObjectMapping.class) {
            return ((PersistentObjectMapping) obj).oid.equals(oid);
        }
        return false;
    }
    
    public String toString() {
        return (specification == null ? "null" : specification.getSingularName()) + " : " + oid + " : " + version;
    }

    public Version getVersion() {
        return version;
    }
    
    public void checkVersion(NakedObject object) {
        object.checkLock(getVersion());
    }
    
    public void updateVersion() {
        NakedObject adapter = NakedObjectsContext.getObjectLoader().getAdapterFor(oid);
        version = adapter.getVersion();
    }
    
    public void restoreToLoader(NakedObjectLoader objectLoader) {
        Oid oid = getOid();
        NakedObject adapter = objectLoader.recreateAdapterForPersistent(oid, specification);
        adapter.setOptimisticLock(getVersion());
    }
}

// Copyright (c) Naked Objects Group Ltd.
