package org.nakedobjects.nos.client.web.request;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.reflect.OneToManyAssociation;
import org.nakedobjects.noa.reflect.ValueAssociation;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nos.client.web.component.Table;

import java.util.Enumeration;


public class TableBuilder {

    public static Table createTable(
            final Context context,
            final String id,
            final NakedObject object,
            final OneToManyAssociation collectionField) {

        NakedCollection collection = (NakedCollection) collectionField.get(object);
        String name = collectionField.getName();
        NakedObjectSpecification type = collectionField.getSpecification();

        String summary = "Table showing elements of " + name + " field";
        return createTable(context, collectionField != null, collection, summary, type);
    }

    public static Table createTable(
            final Context context,
            final boolean addSelector,
            final NakedCollection collection,
            final String summary,
            final NakedObjectSpecification elementType) {
        Enumeration e = collection.elements();

        NakedObjectField[] visibleFields = elementType.getStaticallyVisibleFields();
        NakedObjectField[] fields = new NakedObjectField[visibleFields.length];
        int len = 0;
        for (int i = 0; i < visibleFields.length; i++) {
            if (!visibleFields[i].isCollection()) {
                fields[len++] = visibleFields[i];
            }
        }

        Table table = context.getComponentFactory().createTable(len, addSelector);
        table.setSummary(summary);

        for (int i = 0; i < len; i++) {
            table.addColumnHeader(fields[i].getName());
        }

        while (e.hasMoreElements()) {
            NakedObject element = (NakedObject) e.nextElement();
            NakedObjectsContext.getObjectPersistor().resolveImmediately(element);
            String elementId = context.mapObject(element);
            table.addRowHeader(context.getComponentFactory().createObjectIcon(element, elementId, "icon"));
            for (int i = 0; i < len; i++) {
                Naked fld = fields[i].get(element);
                
                if (!fields[i].isVisible(element)) {
                    table.addEmptyCell();
                } else if (fields[i].isValue()) {
                    table.addCell(fld.titleString(), ((ValueAssociation) fields[i]).getNoLines() > 1);
                } else if (fields[i].isObject() && fld == null) {
                    table.addEmptyCell();
                } else if (fields[i].isObject()) {
                    NakedObjectsContext.getObjectPersistor().resolveImmediately((NakedObject) fld);
                    String objectId = context.mapObject((NakedObject) fld);
                    table.addCell(context.getComponentFactory().createObjectIcon(fields[i], (NakedObject) fld, objectId, "icon"));
                }
            }
            /*
            if (addSelector) {
                table.addCell(context.getFactory().createRemoveOption(id, elementId, collectionField.getId()));
            }
            */
            // TODO add selection box
            // table.addCell();
            /*
             * if (collectionField != null) { if (collectionField.isValidToRemove(object,
             * element).isAllowed()) { table.addCell(context.getFactory().createRemoveOption(id, elementId,
             * collectionField.getId())); } else { table.addEmptyCell(); } }
             */

        }
        return table;
    }

}

// Copyright (c) Naked Objects Group Ltd.
