package org.nakedobjects.nos.client.web.request;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedObjectLoader;
import org.nakedobjects.noa.adapter.Oid;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.Assert;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.util.memento.Memento;


class TransientObjectMapping implements ObjectMapping {
    private final Oid oid;
    private final Memento memento;

    public TransientObjectMapping(final NakedObject object) {
        oid = object.getOid();
        Assert.assertTrue("OID is for persistent", oid.isTransient());
        Assert.assertTrue("adapter is for persistent", object.getResolveState().isTransient());
        memento = new Memento(object);
    }

    public void debug(DebugString debug) {
        memento.debug(debug);
    }

    public Oid getOid() {
        return oid;
    }
    
    public NakedObject getObject() {
        return NakedObjectsContext.getObjectLoader().getAdapterFor(oid);
    }

    public int hashCode() {
        return oid.hashCode();
    }
    
    public boolean equals(Object obj) {
        if (obj instanceof TransientObjectMapping) {
            return ((TransientObjectMapping) obj).oid.equals(oid);
        }
        return false;
    }
    
    public String toString() {
        return "TRANSIENT : " + oid + " : " + memento;
    }
    
    public Version getVersion() {
        return null;
    }
    
    public void checkVersion(NakedObject object) {}
    
    
    public void restoreToLoader(NakedObjectLoader objectLoader) {
        memento.recreateObject();
    }
    
    public void updateVersion() {}

}

// Copyright (c) Naked Objects Group Ltd.
