package org.nakedobjects.nos.client.web.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.nakedobjects.nof.core.conf.Configuration;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.NakedObjectConfiguration;
import org.nakedobjects.nos.client.web.WebController;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.html.HtmlComponentFactory;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Request;


public class Controller extends HttpServlet {
    private static final long serialVersionUID = 1L;
    private final Logger LOG = Logger.getLogger(Controller.class);
    private static final String PROPERTY_BASE = Configuration.ROOT + "viewer.web.";
    private WebController controller;
    private static String encoding = "ISO-8859-1";

    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        request.setCharacterEncoding(encoding);
        processRequest(request, response);
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        processRequest(request, response);
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        LOG.info("request: " + request.getServletPath() + "?" + request.getQueryString());

        Request req = new ServletRequest(request);

        if (req.getRequestType() == null) {
            throw new ServletException("No action specified");
        } else if (!controller.actionExists(req)) {
            throw new ServletException("No such action " + req.getRequestType());
        } else {
            try {
                Context context = getContextForRequest(request);
                processRequest(request, response, req, context);
            } catch (Exception e) {
                LOG.error("exception during request handling", e);
                throw new ServletException("Internal exception", e);
            }
        }
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response, Request req, Context context)
            throws IOException, ServletException {
        if (context.isLoggedIn()) {
            response.setContentType("text/html");

            try {
                SystemAccess.startRequest(context.getSession());
                Page page = controller.generatePage(context, req);
                if (context.isValid()) {
                    if (controller.isDebug()) {
                        controller.addDebug(page, req);
                        addDebug(request, page);
                    }
                    PrintWriter writer;
                    writer = response.getWriter();
                    page.write(writer);
                } else {
                    response.sendRedirect("logon.app");
                }
            } finally {
                SystemAccess.endRequest(context.getSession());
                if (!context.isLoggedIn()) {
                    HttpSession httpSession = request.getSession(false);
                    LOG.info("dropping session: " + httpSession);
                }
            }
        } else {
            response.sendRedirect("logon.app");
        }
    }

    private Context getContextForRequest(HttpServletRequest request) {
        Context context = (Context) request.getSession(true).getAttribute("nof-context");
        if (context == null || !context.isValid()) {
            // TODO reuse the component factory
            context = new Context(new HtmlComponentFactory());
            request.getSession(true).setAttribute("nof-context", context);
        }
        return context;
    }

    private void addDebug(HttpServletRequest request, Page page) {
        page.addDebug("Servlet path", request.getServletPath());
        page.addDebug("Query string", request.getQueryString());
        page.addDebug("Context path", request.getContextPath());
        page.addDebug("Path info", request.getPathInfo());
    }

    public void init(ServletConfig arg0) throws ServletException {
        super.init(arg0);
        controller = new WebController();
        NakedObjectConfiguration configuration = NakedObjectsContext.getConfiguration();
        controller.setDebug(configuration.getBoolean(PROPERTY_BASE + "debug"));
        controller.init();
        encoding = configuration.getString(PROPERTY_BASE + "encoding", encoding);
    }
}

// Copyright (c) Naked Objects Group Ltd.
