package org.nakedobjects.nos.client.web.servlet;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.security.ConnectionManager;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.noa.security.SessionManager;
import org.nakedobjects.nof.core.security.ExplorationSession;
import org.nakedobjects.nof.core.security.PasswordAuthenticationRequest;
import org.nakedobjects.nos.client.web.html.HtmlComponentFactory;
import org.nakedobjects.nos.client.web.request.Context;


public class SystemAccess {

    private static final Logger LOG = Logger.getLogger(SystemAccess.class);
    private static SystemAccess instance;

    // REVIEW part of decoupling systemaccess from servlet code - changed session to object
    public static void addSession(Object session) {
        instance.sessions.add(session);
        LOG.debug("session started " + session);
    }

    public static Session authenticate(PasswordAuthenticationRequest passwordAuthenticationRequest) {
        if (showExploration()) {
            Session session = new ExplorationSession();
            Context context = new Context(new HtmlComponentFactory());
            context.setSession(session);
            return session;
        }

        return getSessionManager().authenticate(passwordAuthenticationRequest);
    }

    public static boolean authenticateUser() {
        return instance.authenticateUser;
    }

    public static void endRequest(final Session session) {
        getManager().disconnect(session);
    }

    public static List getSessions() {
        // REVIEW part of decoupling systemaccess from servlet code - moved list sessions to monitorservlet
        return instance.sessions;
    }

    public static void logoffUser(final Session session) {
        getSessionManager().closeSession(session);
    }

    public static ConnectionManager getManager() {
        if (instance == null) {
            throw new IllegalStateException("Server initialisation failed, or not defined as a context listener");
        }
        return instance.manager;
    }

    public static SessionManager getSessionManager() {
        if (instance == null) {
            throw new IllegalStateException("Server initialisation failed, or not defined as a context listener");
        }
        return instance.sessionManager;
    }

    public static boolean showExploration() {
        return instance.showExploration;
    }

    public static void startRequest(final Session session) {
        getManager().connect(session);
    }

    public static void removeSession(final Object session) {
        LOG.debug("session ended " + session);
        instance.sessions.remove(session);
        // REVIEW part of decoupling systemaccess from servlet code - moved logoff part to sessionlist
    }

    private final ConnectionManager manager;
    private final boolean showExploration;
    private final boolean authenticateUser;
    private final List sessions = new ArrayList();
    private final SessionManager sessionManager;

    public SystemAccess(final ConnectionManager manager, final SessionManager sessionManager) {
        this(manager, sessionManager, true, false);
    }

    public SystemAccess(
            final ConnectionManager manager,
            final SessionManager sessionManager,
            final boolean authenticateUser,
            final boolean showExploration) {
        this.manager = manager;
        this.sessionManager = sessionManager;
        this.authenticateUser = authenticateUser;
        this.showExploration = showExploration;
        SystemAccess.instance = this;
    }
}

// Copyright (c) Naked Objects Group Ltd.
