package org.nakedobjects.nos.client.web.task;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.OneToManyAssociation;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Task;


public class AddItemToCollectionTask extends Task {
    private OneToManyAssociation field;

    protected AddItemToCollectionTask(final Context context, final NakedObject target, final OneToManyAssociation fld) {
        super(context, "Add to collection", "", target, 1);
        names[0] = fld.getName();
        descriptions[0] = fld.getDescription();
        fieldSpecifications[0] = fld.getSpecification();
        initialState[0] = null;
        optional[0] = true;
        // TODO add defaults and options
        this.field = fld;
    }

    public void checkForValidity(Context context) {
        NakedObject target = getTarget(context);
        Naked[] parameters = getEntries(context);

        Consent valueValid = field.isValidToAdd(target, (NakedObject) parameters[0]);
        errors[0] = valueValid.isVetoed() ? valueValid.getReason() : null;
    }

    public void checkInstances(final Context context, final NakedObject[] objects) {
        NakedCollection collection = (NakedCollection) field.get(getTarget(context));
        NakedObject target = getTarget(context);
        for (int i = 0; i < objects.length; i++) {
            Consent valueValid = field.isValidToAdd(target, objects[i]);
            if(valueValid.isVetoed()) {
                objects[i] = null;
            } else if (collection.contains(objects[i])) {
                objects[i] = null;
            }
        }
    }

    public Naked completeTask(Context context, Page page) {
        NakedObject target = getTarget(context);
        Naked[] parameters = getEntries(context);

        NakedObjectsContext.getObjectPersistor().startTransaction();

        field.addElement(target, (NakedObject) parameters[0]);

        if (target.getResolveState().isTransient()) {
            NakedObjectsContext.getObjectPersistor().makePersistent(target);
        }
        NakedObjectsContext.getObjectPersistor().endTransaction();
        return target;
    }

    public boolean isEditing() {
        return true;
    }
}

// Copyright (c) Naked Objects Group Ltd.
