package org.nakedobjects.nos.client.web.task;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.reflect.OneToOneAssociation;
import org.nakedobjects.noa.reflect.ValueAssociation;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Task;


public class EditTask extends Task {
    private static int size(final NakedObject object) {
        NakedObjectField[] fields = object.getSpecification().getDynamicallyVisibleFields(object);
        return fields.length;
    }

    private static boolean skipField(final NakedObject object, final NakedObjectField fld) {
        return fld.isCollection() || fld.isUsable().isVetoed() || fld.isUsable(object).isVetoed();
    }

    private final NakedObjectField[] fields;
    private final String newType;

    EditTask(final Context context, final NakedObject object) {
        super(context, "Edit", "", object, size(object));

        NakedObjectField[] allFields = object.getSpecification().getDynamicallyVisibleFields(object);

        fields = new NakedObjectField[names.length];
        for (int i = 0, j = 0; j < allFields.length; j++) {
            NakedObjectField fld = allFields[j];
            fields[i] = fld;
            names[i] = fld.getName();
            descriptions[i] = fld.getDescription();
            
            Consent usableByUser = fld.isUsable();
            if (usableByUser.isVetoed()) {
                descriptions[i] = usableByUser.getReason();
            } else {
                Consent usableInState = fld.isUsable(object);
                if (usableInState.isVetoed()) {
                    descriptions[i] = usableInState.getReason();
                }
            }
            
            fieldSpecifications[i] = fld.getSpecification();
            initialState[i] = fld.get(object);
            if (skipField(object, fld)) {
                readOnly[i] = true;
            } else {
                readOnly[i] = false;
                optional[i] = ! fld.isMandatory();
                if (fld instanceof ValueAssociation) {
                    noLines[i] = ((ValueAssociation) fld).getNoLines();
                    wraps[i] = ((ValueAssociation) fld).canWrap();
                    maxLength[i] = ((ValueAssociation) fld).getMaximumLength();
                    typicalLength[i] = ((ValueAssociation) fld).getTypicalLineLength();
                }
            }
            i++;
        }
        
        boolean isNew = object.getResolveState() == ResolveState.TRANSIENT;
        newType = isNew ? getTarget(context).getSpecification().getSingularName() : null;
    }

    protected Naked[][] getOptions(Context context, final int from, final int len) {
        NakedObject target = getTarget(context);
        Naked[][] options = new Naked[len][];
        for (int i = from, j = 0; j < len; i++, j++) {
            if (skipField(target, fields[i])) {
            } else {
               options[j] = fields[i].getOptions(target);
            }
        }
        return options;
    }
    
    public void checkForValidity(final Context context) {
        NakedObject target = getTarget(context);
        Naked[] entries = getEntries(context);

        int len = fields.length;
        for (int i = 0; i < len; i++) {
            if (readOnly[i] || errors[i] != null) {
                continue;
            }
            NakedObjectField fld = fields[i];
            if (fld.isValue()) {
                Naked currentValue = fld.get(target);
                NakedValue entryValue = (NakedValue) entries[i];
                if (currentValue == null || currentValue.getObject() == null || ! currentValue.getObject().equals(entryValue.getObject())) {
                    Consent valueValid = ((ValueAssociation) fld).isValueValid(target, (NakedValue) entries[i]);
                    errors[i] = valueValid.isVetoed() ? valueValid.getReason() : null;
                }
            } else if (fld.isObject()) {
                OneToOneAssociation oneToOneAssociation = (OneToOneAssociation) fld;
                NakedObject entryReference = (NakedObject) entries[i];
                Naked currentReference = oneToOneAssociation.get(target);
                if (currentReference != entryReference) {
                    Consent valueValid = ((OneToOneAssociation) fld).isAssociationValid(target, entryReference);
                    errors[i] = valueValid.isVetoed() ? valueValid.getReason() : null;
                }
            }
        }

    }

    public Naked completeTask(final Context context, final Page page) {
        NakedObject target = getTarget(context);
        Naked[] entries = getEntries(context);

        int len = fields.length;
        NakedObjectsContext.getObjectPersistor().startTransaction();
        for (int i = 0; i < len; i++) {
            if(readOnly[i]) {
                continue;
            }
            NakedObjectField fld = fields[i];
            if (fld.isValue()) {
                ValueAssociation valueAssociation = ((ValueAssociation) fld);
                NakedValue entryValue = (NakedValue) entries[i];
                if (entryValue.getObject() == null) {
                    if (!valueAssociation.canClear()) {
                        errors[i] = "Can't clear this value";
                    } else if (!valueAssociation.isEmpty(target)) {
                        valueAssociation.clearValue(target);    
                    }
                } else {
                    Naked currentValue = valueAssociation.get(target);
                    if (currentValue == null || currentValue.getObject() == null || ! currentValue.getObject().equals(entryValue.getObject())) {
                        valueAssociation.setValue(target, entryValue);
                    }
                }
            } else if (fld.isObject()) {
                OneToOneAssociation oneToOneAssociation = (OneToOneAssociation) fld;
                NakedObject entryReference = (NakedObject) entries[i];
                Naked currentReference = oneToOneAssociation.get(target);
                if (currentReference != entryReference) {
                    oneToOneAssociation.setAssociation(target, entryReference);
                }
            }
        }
        if (target.getResolveState().isTransient()) {
            NakedObjectAction action = target.getSpecification().getObjectAction(NakedObjectAction.USER, "save",
                    new NakedObjectSpecification[0]);
            if (action == null) {
                NakedObjectsContext.getObjectPersistor().makePersistent(target);
            } else {
                action.execute(target, new Naked[0]);
            }
        }
        NakedObjectsContext.getObjectPersistor().endTransaction();

        return target;
    }

    protected boolean simpleField(final NakedObjectSpecification type, final int i) {
        return !fields[i].isOptionEnable() || super.simpleField(type, i);
    }
    
    public boolean isEditing() {
        return true;
    }
    
    public String getName() {
        if (newType == null) {
            return super.getName();
        } else {
            return "New " + newType;
        }
    }
}

// Copyright (c) Naked Objects Group Ltd.
