package org.nakedobjects.nos.client.web.task;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.spec.Features;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.UnknownTypeException;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.request.Action;
import org.nakedobjects.nos.client.web.request.ActionException;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.ForwardRequest;
import org.nakedobjects.nos.client.web.request.Request;
import org.nakedobjects.object.MessageBroker;


public final class InvokeMethod implements Action {

    public void execute(Request request, Context context, Page page) {
        String idString = request.getObjectId();
        if (idString == null) {
            throw new ActionException("Task no longer in progress");
        }
        NakedObject target = context.getMappedObject(idString);
        String id = request.getActionId();
        NakedObjectAction action = context.getMappedAction(id);
        if (action == null) {
            throw new ActionException("No such action: " + action.getName());
        }

        boolean executeImmediately = false;
        boolean isContributedMethod = action.isContributedMethodWithSuitableParameter();
        if (action.getParameterCount() == 0) {
            executeImmediately = true;
        } else if (action.getParameterCount() == 1 && isContributedMethod
                && target.getSpecification().isOfType(action.getParameterTypes()[0])) {
            executeImmediately = true;
        }

        if (executeImmediately) {
            Naked[] parameters = isContributedMethod ? new Naked[] { target } : null;
            Naked result = action.execute(target, parameters);
            MessageBroker broker = NakedObjectsContext.getMessageBroker();
            String[] messages = broker.getMessages();
            String[] warnings = broker.getWarnings();
            context.setMessagesAndWarnings(messages, warnings);
            String targetId = context.mapObject(target);
            displayMethodResult(request, context, page, result, targetId);
        } else {
            MethodTask methodTask = new MethodTask(context, target, action);
            context.addTaskCrumb(methodTask);
            request.forward(ForwardRequest.task(methodTask));
        }
    }

    static void displayMethodResult(
            final Request request,
            final Context context,
            final Page page,
            final Naked result,
            final String targetId) {
        if (result == null) {
            // TODO ask context for page to display - this will be the most recent object prior to the task
            // null object - so just view service 
            request.forward(ForwardRequest.viewService(targetId));
        } else {
            if (result.getSpecification().getType() == NakedObjectSpecification.COLLECTION) {
                NakedCollection coll = (NakedCollection) result;
                if (coll.size() == 1) {
                    forwardResult(request, context, coll.firstElement());
                } else {
                    forwardResult(request, context, coll);
                }
            } else if (result.getSpecification().getType() == NakedObjectSpecification.OBJECT) {
                forwardResult(request, context, (NakedObject) result);
            } else {
                throw new UnknownTypeException(result.getSpecification().getFullName());
            }
        }
    }

    static void forwardResult(final Request request, final Context context, final NakedCollection coll) {
        String collectionId = context.mapCollection(coll);
        request.forward(ForwardRequest.listCollection(collectionId));
    }

    static void forwardResult(final Request request, final Context context, final NakedObject result) {
        String objectId = context.mapObject(result);
        if (result.getResolveState().isTransient() && result.persistable() == Persistable.USER_PERSISTABLE) {
            request.forward(ForwardRequest.editObject(objectId));
        } else if (Features.isService(result.getSpecification())) {
            request.forward(ForwardRequest.viewService(objectId));
        } else {
            request.forward(ForwardRequest.viewObject(objectId));
        }
    }

    public String name() {
        return "method";
    }

}

// Copyright (c) Naked Objects Group Ltd.
