package org.nakedobjects.nos.client.web.task;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Task;
import org.nakedobjects.object.MessageBroker;

public final class MethodTask extends Task {
    private NakedObjectAction action;

    protected MethodTask(final Context context, final NakedObject target, final NakedObjectAction action) {
        super(context, action.getName(), action.getDescription(), target, action.getParameterCount()) ; 
        this.action = action;
        
        String[] names = action.getParameterNames();
        String[] descriptions = action.getParameterDescriptions();
        NakedObjectSpecification[] types = action.getParameterTypes();
        Naked[] defaultParameterValues = action.getDefaultParameterValues(target);
        boolean[] optional = action.getOptionalParameters();
        for (int i = 0; i < names.length; i++) {
            this.names[i] = names[i];
            this.descriptions[i] = descriptions[i];
            this.fieldSpecifications[i] = types[i];
            this.optional[i] = optional[i];
            if(defaultParameterValues[i] == null) {
                if (action.isContributedMethodWithSuitableParameter() && target.getSpecification().isOfType(types[i])) {
                    initialState[i] = target;
                } else {
                    initialState[i] = null;
                }
            } else {
                initialState[i] = defaultParameterValues[i];
            }
            noLines[i] = action.getParameterNoLines()[i];
            wraps[i] = action.canParametersWrap()[i];
            maxLength[i] = action.getParameterMaxLengths()[i];
            typicalLength[i] = action.getParameterTypicalLengths()[i];           
        }
        
        
    }
    
    public void checkForValidity(final Context context) {
        Naked[] parameters = getEntries(context);
        NakedObject target = getTarget(context);
        Consent consent = action.isParameterSetValid(target, parameters);
        error = null;
        if(consent.isVetoed()) {
            error = consent.getReason();
        }
    }
    
    public Naked completeTask(Context context, Page page) {
        Naked[] parameters = getEntries(context);
        NakedObject target = getTarget(context);
        Naked result = action.execute(target, parameters );
        MessageBroker broker = NakedObjectsContext.getMessageBroker();
        String[] messages = broker.getMessages();
        String[] warnings = broker.getWarnings();
        context.setMessagesAndWarnings(messages, warnings);
        return result;
    }
    
    public void debug(DebugString debug) {
        debug.appendln("action: " + action);
        super.debug(debug);
    }

    protected Naked[][] getOptions(Context context, int from, int len) {
        Naked[][] allOptions = action.getOptions(getTarget(context));
        Naked[][] options = new Naked[len][];
        for (int i = from, j = 0; j < len; i++, j++) {
            options[j] = allOptions[i];
        }
        return options;
    }
    
    public boolean collectParameters() {
        int expectedNoParameters = action.isContributedMethodWithSuitableParameter() ? 1 : 0;
        return action.getParameterCount() == expectedNoParameters;
    }
}


// Copyright (c) Naked Objects Group Ltd.
