package org.nakedobjects.nos.client.web.task;

import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nos.client.web.component.Component;
import org.nakedobjects.nos.client.web.component.Form;
import org.nakedobjects.nos.client.web.component.Page;
import org.nakedobjects.nos.client.web.component.ViewPane;
import org.nakedobjects.nos.client.web.request.Action;
import org.nakedobjects.nos.client.web.request.ActionException;
import org.nakedobjects.nos.client.web.request.Context;
import org.nakedobjects.nos.client.web.request.Request;
import org.nakedobjects.nos.client.web.request.Task;
import org.nakedobjects.nos.client.web.request.TaskLookupException;


public final class TaskStep implements Action {
    private void addSelector(
            final Context context,
            final Form form,
            final String currentEntry,
            final String fieldId,
            final String fieldLabel,
            final String fieldDescription,
            final boolean required,
            final String errorMessage,
            final Task task,
            NakedObject[] objects) {
        task.checkInstances(context, objects);

        int size = 0;
        for (int i = 0; i < objects.length; i++) {
            if (objects[i] != null) {
                size++;
            }
        }

        String[] instances = new String[size];
        String[] ids = new String[size];
        int selectedIndex = -1;

        for (int i = 0, j = 0; i < objects.length; i++) {
            NakedObject element = objects[i];
            if (element != null) {
                instances[j] = element.titleString();
                ids[j] = context.mapObject(element);
                if (ids[j].equals(currentEntry)) {
                    selectedIndex = i;
                }
                j++;
            }
        }
        form.addLookup(fieldLabel, fieldDescription, fieldId, selectedIndex, instances, ids, required, errorMessage);
    }

    private void addSelectorForKnownReferences(
            final Context context,
            final Form form,
            final NakedObjectSpecification type,
            final String currentEntry,
            final String fieldId,
            final String fieldLabel,
            final String fieldDescription,
            final boolean required,
            final String errorMessage,
            final Task task) {

        NakedObject[] objects = context.getKnownInstances(type);
        addSelector(context, form, currentEntry, fieldId, fieldLabel, fieldDescription, required, errorMessage, task, objects);
    }

    private void addSelectorForObjectOptions(
            final Context context,
            final Form form,
            final String currentEntry,
            final String fieldId,
            final String fieldLabel,
            final String fieldDescription,
            final Naked[] options,
            final boolean required,
            final String errorMessage,
            final Task task) {
        NakedObject[] objects = new NakedObject[options.length];
        for (int i = 0; i < options.length; i++) {
            objects[i] = (NakedObject) options[i];
        }
        addSelector(context, form, currentEntry, fieldId, fieldLabel, fieldDescription, required, errorMessage, task, objects);
    }

    private void addSelectorForValueOptions(
            final Form form,
            final String currentEntry,
            final String fieldId,
            final String fieldLabel,
            final String fieldDescription,
            final Naked[] options,
            final boolean required,
            final String errorMessage,
            final Task task) {
        int selectedIndex = -1;
        String[] instances = new String[options.length];
        for (int i = 0; i < options.length; i++) {
            instances[i] = options[i].titleString();
            if (currentEntry.equals(instances[i])) {
                selectedIndex = i;
            }
        }
        form.addLookup(fieldLabel, fieldDescription, fieldId, selectedIndex, instances, instances, required, errorMessage);
    }

    private void addTextFieldForValue(
            final Form form,
            final NakedObjectSpecification type,
            final String currentEntryText,
            final String fieldId,
            final String fieldLabel,
            final String fieldDescription,
            final int noLines,
            final boolean wrap,
            final int maxLength,
            final int typicalLength,
            final boolean required,
            final String errorMessage) {
        form.addField(type, fieldLabel, fieldDescription, fieldId, currentEntryText, noLines, wrap, maxLength, typicalLength,
                required, errorMessage);
    }


    private void displayTask(final Context context, final Page page, final Task task) {
        page.setTitle(task.getName());

        ViewPane content = page.getViewPane();
        NakedObject target = task.getTarget(context);
        String titleString = target.titleString();
        if (target.getResolveState().isTransient()) {
            titleString += " (Unsaved)";
        }
        content.setTitle(titleString, target.getSpecification().getDescription());
        String iconName = target.getIconName();
        if (iconName == null) {
            iconName = target.getSpecification().getShortName();
        }
        content.setIconName(iconName);

        StringBuffer crumbs = new StringBuffer();
        String[] trail = task.getTrail();
        for (int i = 0; i < trail.length; i++) {
            crumbs.append(" : ");
            crumbs.append(trail[i]);
        }

        Component[] action = new Component[1];
        action[0] = context.getComponentFactory().createInlineBlock("name", task.getName(), task.getDescription());
        content.setMenu(action);

        if (task.getError() != null) {
            content.add(context.getComponentFactory().createInlineBlock("error", task.getError(), null));
        }

        Form form = context.getComponentFactory().createForm(task.getId(), name(), task.getStep(), task.numberOfSteps(), task.isEditing());
        String[] parameterLabels = task.getNames();
        String[] parameterDescriptions = task.getFieldDescriptions();
        String[] errors = task.getErrors();
        String[] entryText = task.getEntryText();
        int[] noLines = task.getNoLines();
        boolean[] canWrap = task.getWraps();
        int[] maxLength = task.getMaxLength();
        int[] typicalLength = task.getTypicalLength();
        Naked[][] options = task.getOptions(context);
        boolean[] optional = task.getOptional();
        boolean[] readOnly = task.getReadOnly();
        NakedObjectSpecification[] types = task.getTypes();
        for (int i = 0; i < parameterLabels.length; i++) {
            NakedObjectSpecification type = types[i];
            String fieldId = "fld" + i;
            String fieldLabel = parameterLabels[i] == null ? "" : parameterLabels[i];;
            String fieldDescription = parameterDescriptions[i] == null ? "" : parameterDescriptions[i];
            String currentEntryTitle = entryText[i];
            String error = errors[i];
            int t = type.getType();
            if (readOnly[i]) {
                addReadOnlyField(form, fieldLabel, currentEntryTitle, fieldDescription);
            } else if (t == NakedObjectSpecification.VALUE && options[i] != null && options[i].length > 0) {
                addSelectorForValueOptions(form, currentEntryTitle, fieldId, fieldLabel, fieldDescription, options[i],
                        !optional[i], error, task);
            } else if (t == NakedObjectSpecification.VALUE) {
                addTextFieldForValue(form, type, currentEntryTitle, fieldId, fieldLabel, fieldDescription, noLines[i], canWrap[i],
                        maxLength[i], typicalLength[i], !optional[i], error);
            } else if (t == NakedObjectSpecification.OBJECT && options[i] != null && options[i].length > 0) {
                addSelectorForObjectOptions(context, form, currentEntryTitle, fieldId, fieldLabel, fieldDescription, options[i],
                        !optional[i], error, task);
            } else if (t == NakedObjectSpecification.OBJECT) {
                addSelectorForKnownReferences(context, form, type, currentEntryTitle, fieldId, fieldLabel, fieldDescription,
                        !optional[i], error, task);
            } else {
                throw new NakedObjectRuntimeException();
            }
        }
        content.add(form);
    }

    private void addReadOnlyField(final Form form, final String fieldLabel, final String title, final String description) {
        form.addReadOnlyField(fieldLabel, title, description);
    }

    public void execute(final Request request, final Context context, final Page page) {
        String taskId = request.getTaskId();
        Task task = context.getTask(taskId);
        String button = request.getButtonName();
        if (task == null && !"Cancel".equals(button)) {
            throw new TaskLookupException("No task found with id " + taskId);
        }

        if (button == null) {
            // start new task
            displayTask(context, page, task);
        } else if ("Cancel".equals(button)) {
            forwardCancel(request, context, task);
        } else if ("Previous".equals(button)) {
            task.setFromFields(request, context);
            task.previousStep();
            displayTask(context, page, task);
        } else if ("Next".equals(button)) {
            task.setFromFields(request, context);
            task.nextStep();
            displayTask(context, page, task);
        } else if ("Finish".equals(button) || "Save".equals(button) || "Ok".equals(button)) {
            task.setFromFields(request, context);
            task.checkForValidity(context);

            if (hasErrors(task)) {
                displayTask(context, page, task);
            } else {
                String targetId = context.mapObject(task.getTarget(context));
                Naked result = task.completeTask(context, page);
                if (result instanceof NakedObject) {
                    NakedObject object = (NakedObject) result;
                    context.updateVersion(object);
                }
                InvokeMethod.displayMethodResult(request, context, page, result, targetId);
                context.endTask(task);
            }
        } else {
            throw new ActionException("No task action: " + button);
        }
    }

    private void forwardCancel(final Request request, final Context context, final Task task) {
        Request cancelTask = context.cancelTask(task);
        request.forward(cancelTask);
    }

    private boolean hasErrors(final Task task) {
        if (task.getError() != null) {
            return true;
        }
        String[] errors = task.getErrors();
        for (int i = 0; i < errors.length; i++) {
            if (errors[i] != null) {
                return true;
            }
        }
        return false;
    }

    public String name() {
        return Request.TASK_COMMAND;
    }
}

// Copyright (c) Naked Objects Group Ltd.
